##################################################################################
# Initialize the default script exit code.
$exitCode = 1
##################################################################################
# Output execution parameters.
Write-Verbose "Executing TcmExecvNext with the following parameters:" -Verbose
Write-Verbose "Build Directory: $BuildDirectory" -Verbose
Write-Verbose "Test Environment: $TestEnvironment" -Verbose
Write-Verbose "Collection URL: $TfsUrl" -Verbose
Write-Verbose "Team project: $TeamProject" -Verbose
Write-Verbose "Plan ID: $PlanId" -Verbose
Write-Verbose "Suite ID: $SuiteId" -Verbose
Write-Verbose "Configuration ID: $ConfigId" -Verbose
Write-Verbose "Title: $Title" -Verb
##################################################################################
# Define globally used variables and constants.
[Int16]$TestRunWaitDelay = 10
# Visual Studio 2013
$vscommtools = [System.Environment]::GetEnvironmentVariable("VS120COMNTOOLS")
$tcmExe = [System.IO.Path]::GetFullPath($vscommtools + "..\IDE\TCM.exe")

##################################################################################
# Ensure TCM.EXE is available in the assumed path.
if ([System.IO.File]::Exists($tcmExe))
{
    ##################################################################################
    # Create the test run.
    "`nCreating test run ..."
    $testRunId = & "$tcmExe" run /create /title:"$Title" /planid:$PlanId /suiteid:$SuiteId /configid:$ConfigId /collection:"$TfsUrl" /teamproject:"$TeamProject" /testenvironment:"$TestEnvironment" /builddir:"$BuildDirectory" /include
    if ($testRunId -match '.+\:\s(?<TestRunId>\d+)\.')
    {
        # The test run ID is identified as a property in the match collection
        # so we can access it directly by using the group name from the regular
        # expression (i.e. TestRunId).
        $testRunId = $matches.TestRunId

        "Waiting for test run $testRunId to complete ..."
        $waitingForTestRunCompletion = $true
        while ($waitingForTestRunCompletion)
        {
            Start-Sleep -s $TestRunWaitDelay
			# We check if the test run is finished by ensuring the date completed is set. It is not possible to do a "is not null" in WIQL so we use a dummy low date to compare it.
            $testRunStatus = & "$tcmExe" run /list /collection:"$TfsUrl" /teamproject:"$TeamProject" /querytext:"SELECT * FROM TestRun WHERE TestRunId=$testRunId and CompleteDate>'1900-01-01'"
			# If there is more than 2 lines, it means there was a result to the query, meaning the run is completed.
            if ($testRunStatus.Count -gt 2)
            {
                $waitingForTestRunCompletion = $false
            }
        }

        "Evaluating test run $testRunId results..."
        # We do a small pause since the results might not be published yet.
        Start-Sleep -s $TestRunWaitDelay

        $testRunResultsTrxFileName = "TestRunResults$testRunId.trx"
        & "$tcmExe" run /export /id:$testRunId /collection:"$TfsUrl" /teamproject:"$TeamProject" /resultsfile:"$testRunResultsTrxFileName" | Out-Null
        if ([System.IO.File]::Exists($testRunResultsTrxFileName))
        {
            # Load the XML document contents.
            [xml]$testResultsXml = Get-Content "$testRunResultsTrxFileName"
            
            # Extract the results of the test run.
            $total = $testResultsXml.TestRun.ResultSummary.Counters.total
            $passed = $testResultsXml.TestRun.ResultSummary.Counters.passed
            $failed = $testResultsXml.TestRun.ResultSummary.Counters.failed
            $inconclusive = $testResultsXml.TestRun.ResultSummary.Counters.inconclusive

            # Output the results of the test run.
            "`n========== Test: $total tests ran, $passed succeeded, $failed failed, $inconclusive inconclusive =========="

            # Determine if there were any failed tests during the test run execution.
            if ($failed -eq 0 -and (-not $InconclusiveFailsTests -or $inconclusive -eq 0))
            {
                # Update this script's exit code.
                $exitCode = 0
            }

            # Remove the test run results file.
            [System.IO.File]::Delete($testRunResultsTrxFileName) | Out-Null
        }
        else
        {
            "`nERROR: Unable to export test run results file for analysis."
        }
    }
}
else
{
    "`nERROR: Unable to locate $tcmExe"
}

##################################################################################
# Indicate the resulting exit code to the calling process.
if ($exitCode -gt 0)
{
    "`nERROR: Operation failed with error code $exitCode."
}
"`nDone."
exit $exitCode